#!/bin/env python3
# coding:utf-8
"""
    修正Zabbix 中 Discovered hosts 组中以下划线和数字结尾的 Hostname
"""

import sys
import re
import argparse

parser = argparse.ArgumentParser("Correct discoverd host name")
parser.set_defaults(handler=lambda args: main(args))

def main(args):

    pattern = re.compile(r'_\d+$')

    zapi = args.zapi
    #获取 Discovered hosts 组的group id
    group = zapi.hostgroup.get({
        'output': ['groupid'],
        'filter': {'name': 'Discovered hosts'}
    })

    if group:
        group_id = group[0]['groupid']
    else:
        sys.exit(-1)

    # 获取属于 Discovered hosts 组的 Host
    hosts = zapi.host.get({
        'output': ['hostid', 'host'],
        'groupids': [group_id]
    })

    for host in hosts:

        # 过滤出Host name是_x 结尾, 并修改 Host name
        if pattern.search(host['host']):

            new_host_name = pattern.sub('', host['host'])
            zapi.host.update({
                'hostid': host['hostid'],
                'host': new_host_name
            })
            print(f'update host name success: hostid={host["hostid"]}, hostname {host["host"]} => {new_host_name}')



#!/usr/bin/python3
"""
在zabbix上依据zabbix主机组生成it-service树
"""
import argparse

parser = argparse.ArgumentParser('Create or delete zabbix service tree')
parser.add_argument('action', choices=['create', 'delete'], help="Create\Delete IT service Tre")
parser.add_argument('-n', '--service-name', required=True, help="The Name of IT service Tree's root")
parser.add_argument('-g', '--group-name', help="Create IT service tree from the Group")
parser.set_defaults(handler=lambda args: main(args))


class CreateItService:
    def __init__(self, zapi, service_name, group_name):
        self.zapi = zapi
        self.service_name = service_name
        self.group_name = group_name

    def create(self):
        # Get root service
        rootsrv = self.zapi.service.get({'filter': {'name': self.service_name}, 'parentids': None})
        if len(rootsrv):
            print(f'The root service {self.service_name} already exists, exit with nothing to do')
            return

        # Create root it service
        rootsrv = self.zapi.service.create({
                    'name': self.service_name,
                    'algorithm': 1,
                    'showsla': 1,
                    'goodsla': 99.99,
                    'sortorder': 0
                 })
        groups = self.zapi.hostgroup.get({
                    'filter': {'name': self.group_name},
                    'output': ['groupid']
                 })
        hosts = self.zapi.host.get({
                    'groupids': [group['groupid'] for group in groups],
                    'selectTriggers': 'extend'
                })
        for host in hosts:
            # Create host level it service
            hostsrv = self.zapi.service.create({
                        'name': '{0} {1}'.format(self.service_name, host['name']),
                        'algorithm': 1,
                        'showsla': 0,
                        'goodsla': 99.99,
                        'sortorder': 0,
                        'parentid': rootsrv['serviceids'][0]
                      })

            for trigger in host.get('triggers'):
                # Create trigger level it service
                self.zapi.service.create({
                    'name': trigger['description'],
                    'triggerid': trigger['triggerid'],
                    'algorithm': 1,
                    'showsla': 1,
                    'goodsla': 99.99,
                    'sortorder': 0,
                    'parentid': hostsrv['serviceids'][0]
                })


class DeleteItService:

    def __init__(self, zapi, service_name):
        # super().__init__(zapi)
        self.zapi = zapi
        self.service_name = service_name

    def hard_service_delete(self, service):
        """
        Delete it service and it's dependencies recursively
        """
        # First delete the dependencies of the service
        for dependency in service.get('dependencies', []):
            tmpsrvs = self.zapi.service.get({
                        'serviceids': dependency['serviceid'],
                        'selectDependencies': 'extend'
                      })
            for tmpsrv in tmpsrvs:
                self.hard_service_delete(tmpsrv)

        # The dependencies of the service had deleted, Now delete the servie
        self.zapi.service.delete([service['serviceid']])

    def delete(self):
        # Get the root it service with specifed service name
        rootsrvs = self.zapi.service.get({
                    'filter': {'name': self.service_name},
                    'parentids': None,
                    'selectDependencies': 'extend'
                   })
        for rootsrv in rootsrvs:
            print(f'The root it service {self.service_name} and all of dependencies will be deleted')
            self.hard_service_delete(rootsrv)


def main(args):

    def create(args):
        CreateItService(args.zapi, args.service_name, args.group_name).create()

    def delete(args):
        DeleteItService(args.zapi, args.service_name).delete()

    f = locals().get(args.action)
    f(args)



from argparse import Namespace
import imp
import multicommand
from zabbix_api import ZabbixAPI
from . import commands


def main():
    parser = multicommand.create_parser(commands)
    parser.add_argument('-s', '--zbx-server', required=True, help='URL of zabbix server')
    parser.add_argument('-u', '--zbx-user', required=True, help='Zabbix server login username')
    parser.add_argument('-p', '--zbx-passwd', required=True, help='Zabbix server login password')
    args = parser.parse_args()

    zapi = ZabbixAPI(args.zbx_server, timeout=60)
    zapi.validate_certs = False
    zapi.login(user=args.zbx_user, password=args.zbx_passwd)

    setattr(args, 'zapi', zapi)

    if hasattr(args, "handler"):
        args.handler(args)

    zapi.logout()

if __name__ == "__main__":
    exit(main())


#! /usr/bin/env python3
# -*- coding: UTF-8 -*-

import argparse
import time
import re
from datetime import datetime
from functools import reduce
from collections.abc import Iterable
from elasticsearch import Elasticsearch, helpers

parser = argparse.ArgumentParser('Gather zabbix host informations and create es index')
parser.add_argument('--es_url', required=True, help="ElasticSearch server ip")
parser.add_argument('--es_user', default='', help="ElasticSearch server login user")
parser.add_argument('--es_passwd', default='', help="ElasticSearch server login password")
parser.set_defaults(handler=lambda args: main(args))

class ItemFilterAndAccumulator(object):

    def __init__(self, keywords):
        if not isinstance(keywords, Iterable):
            raise ValueError('keyworks must be iterabled')
        self.keywords = keywords

    def is_keywords(self, item):
        for word in self.keywords:
            if word not in item['key_']:
                return False
        return True

    def get_result(self, items):
        filtered_items = filter(self.is_keywords, items)
        result = reduce(lambda x, y: x+y, [
                        item['lastvalue'] for item in filtered_items], 0)
        return result if result > 0 else None


def format_items(host):
    if host['flags'] != "4":
        for item in host['items']:
            item['lastvalue'] = float(
                item['lastvalue']) if (item['value_type'] == '0' or item['value_type'] == '3') else item['lastvalue']
        host['filesystems_total'] = ItemFilterAndAccumulator(
            ['vfs.fs.size', 'total']).get_result(host['items'])
        host['filesystems_used'] = ItemFilterAndAccumulator(
            ['vfs.fs.size', 'used']).get_result(host['items'])
        host['memory_total'] = ItemFilterAndAccumulator(
            ['vm.memory.size', 'total']).get_result(host['items'])
        host['memory_used'] = ItemFilterAndAccumulator(
            ['vm.memory.size', 'used']).get_result(host['items'])
    host.pop('items')


# linux 网络接口 ip, 后面却没用到, 不知原因, 先注释
#linux_match = re.compile(
#    r'^ *inet (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}).*$', re.M)

# windows 网络接口 ip, 后面却没用到, 不知原因, 先注释
#win_match = re.compile(
#    r'^Ethernet *enabled *(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}).*$', re.M)

def get_hosts(zapi, es):

    body_datas = []

    hosts = zapi.host.get({
        'output': 'extend',
        'selectGroups': 'extend',
        'selectInterfaces': 'extend',
        'selectInventory': 'extend',
        'selectItems': ['name', 'lastvalue', 'value_type', 'key_'],
    })
    dt = time.strftime('%Y.%m.%d', time.localtime())

    for host in hosts:
        host['@timestamp'] = datetime.utcfromtimestamp(time.time())
        format_items(host)

        host['group_names'] = [group['name'] for group in host['groups']]

# 这段代码, 后面却没用到, 不知原因, 先注释
#        ipv4_addresses = linux_match.findall(
#            host['inventory'].get('host_networks', ''), re.M)
#
#        ipv4_addresses = win_match.findall(
#            host['inventory'].get('host_networks', ''), re.M)
#
#        if "127.0.0.1" in ipv4_addresses:
#            ipv4_addresses.remove("127.0.0.1")
#        host['ipv4_addresses'] = ipv4_addresses

        body_datas.append({
            '_id': host['hostid'],
            '主机名称':  host['inventory'].get('name', host['host']),
            '主机别名': host['inventory'].get('alias', host['host']),
            '接口地址': [aif['ip'] for aif in host['interfaces']],
            '主机组': [grp['name'] for grp in host['groups']],
            'OS': host['inventory'].get('os'),
            'OS_FULL': host['inventory'].get('os_full'),
            'OS_SHORT': host['inventory'].get('os_short'),
            '资产标签': host['inventory'].get('asset_tag'),
            '主负责人': host['inventory'].get('poc_1_name'),
            '次负责人': host['inventory'].get('poc_2_name'),
            '机架': host['inventory'].get('chassis'),
            '子网掩码': host['inventory'].get('host_netmask'),
            '主机网络': host['inventory'].get('host_networks'),
            '机房': host['inventory'].get('location'),
            '机柜': host['inventory'].get('site_rack'),
            '序列号一': host['inventory'].get('serialno_a'),
            '序列号二': host['inventory'].get('serialno_b'),
            'MAC_A': host['inventory'].get('macaddress_a'),
            'MAC_B': host['inventory'].get('macaddress_b'),
            '硬件架构': host['inventory'].get('hw_arch'),
            '标签':  host['inventory'].get('tag'),
            '类型': host['inventory'].get('type'),
            '具体类型': host['inventory'].get('type_full'),
            '型号': host['inventory'].get('model'),
            '供应商': host['inventory'].get('vendor'),
            '@timestamp': datetime.utcfromtimestamp(time.time())
        })

    for host in hosts:
        host['_id'] = host['hostid']
    helpers.bulk(es, hosts, index='zabbix-raw-host-info-' + dt, raise_on_error=True)
    helpers.bulk(es, body_datas, index='zabbix-host-info-' + dt, raise_on_error=True)


def main(args):
    zapi = args.zapi
    es = Elasticsearch(args.es_url, http_auth=(args.es_user, args.es_passwd))
    get_hosts(zapi, es)



from setuptools import setup, find_packages

with open("requirements.txt") as reqs_file:
    reqs = reqs_file.readlines()

setup(
    name ="zbxtool-cfomp",
    version ="0.4.5",
    packages = find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires = reqs,

    entry_points={
        'console_scripts':[
            'zbxtool = lib.cli:main'
        ],
    },
)

